@@ -648,6 +648,13 @@ options:
         existing container.
     type: bool
     default: False
+  requires:
+    description:
+      - Specify one or more requirements. A requirement is a dependency
+        container that will be started before this container.
+        Containers can be specified by name or ID.
+    type: list
+    elements: str
   restart_policy:
     description:
       - Restart policy to follow when containers exit.
@@ -674,7 +681,9 @@ options:
     type: bool
   secrets:
     description:
-      - Add the named secrets into the container at /run/secrets.
+      - Add the named secrets into the container.
+        The format is C(secret[,opt=opt...]), see
+        L(documentation,https://docs.podman.io/en/latest/markdown/podman-run.1.html#secret-secret-opt-opt) for more details.
     type: list
     elements: str
   security_opt:
