@@ -1,4 +1,4 @@
-# (c) 2021, NetApp, Inc
+# (c) 2022, NetApp, Inc
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
 ''' unit tests Cloudmanager Ansible module: '''
@@ -71,12 +71,16 @@ class TestMyModule(unittest.TestCase):
             'name': 'TestA',
             'client_id': 'test',
             'location': 'westus',
+            'use_latest_version': False,
+            'ontap_version': 'ONTAP-9.10.0.T1.azure',
             'vnet_id': 'vpc-test',
             'resource_group': 'test',
             'subnet_id': 'subnet-test',
             'subscription_id': 'test',
             'cidr': '10.0.0.0/24',
             'svm_password': 'password',
+            'license_type': 'azure-cot-standard-paygo',
+            'instance_type': 'Standard_DS4_v2',
             'refresh_token': 'myrefresh_token',
             'is_ha': False
         })
@@ -113,6 +117,24 @@ class TestMyModule(unittest.TestCase):
             'is_ha': False
         })
 
+    def set_args_create_bynode_cloudmanager_cvo_azure(self):
+        return dict({
+            'state': 'present',
+            'name': 'Dummyname',
+            'client_id': 'test',
+            'location': 'westus',
+            'vnet_id': 'vpc-test',
+            'resource_group': 'test',
+            'subscription_id': 'test',
+            'cidr': '10.0.0.0/24',
+            'subnet_id': 'subnet-test',
+            'svm_password': 'password',
+            'refresh_token': 'myrefresh_token',
+            'license_type': 'azure-cot-premium-byol',
+            'serial_number': '12345678',
+            'is_ha': False
+        })
+
     def test_module_fail_when_required_args_missing(self):
         ''' required arguments are reported as errors '''
         with pytest.raises(AnsibleFailJson) as exc:
@@ -221,9 +243,7 @@ class TestMyModule(unittest.TestCase):
     def test_create_cloudmanager_cvo_azure_nodebase_license_pass(self, get_post_api,
                                                                  get_working_environment_details_by_name, get_nss,
                                                                  get_tenant, wait_on_completion, get_token):
-        data = self.set_args_create_cloudmanager_cvo_azure()
-        data['license_type'] = 'azure-cot-premium-byol'
-        data['serial_number'] = '12345678'
+        data = self.set_args_create_bynode_cloudmanager_cvo_azure()
         set_module_args(data)
         get_token.return_value = 'test', 'test'
         my_obj = my_module()
@@ -249,7 +269,7 @@ class TestMyModule(unittest.TestCase):
     def test_create_cloudmanager_cvo_azure_ha_nodebase_license_pass(self, get_post_api,
                                                                     get_working_environment_details_by_name, get_nss,
                                                                     get_tenant, wait_on_completion, get_token):
-        data = self.set_args_create_cloudmanager_cvo_azure()
+        data = self.set_args_create_bynode_cloudmanager_cvo_azure()
         data['is_ha'] = True
         data['license_type'] = 'azure-ha-cot-premium-byol'
         data['platform_serial_number_node1'] = '12345678'
@@ -280,6 +300,8 @@ class TestMyModule(unittest.TestCase):
                                                    get_tenant, wait_on_completion, get_token):
         data = self.set_args_create_cloudmanager_cvo_azure()
         data['is_ha'] = True
+        data['license_type'] = 'ha-capacity-paygo'
+        data['capacity_package_name'] = 'Essential'
         set_module_args(data)
         get_token.return_value = 'test', 'test'
         my_obj = my_module()
@@ -319,32 +341,53 @@ class TestMyModule(unittest.TestCase):
         assert exc.value.args[0]['changed']
 
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp.CloudManagerRestAPI.get_token')
+    @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.update_instance_license_type')
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.update_tier_level')
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.update_cvo_tags')
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.update_svm_password')
+    @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.upgrade_ontap_image')
+    @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.get_working_environment_details')
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.get_working_environment_property')
     @patch('ansible_collections.netapp.cloudmanager.plugins.module_utils.netapp_module.NetAppModule.get_working_environment_details_by_name')
-    def test_change_cloudmanager_cvo_azure(self, get_cvo, get_property, update_svm_password, update_cvo_tags,
-                                           update_tier_level, get_token):
-        set_module_args(self.set_default_args_pass_check())
+    def test_change_cloudmanager_cvo_azure(self, get_cvo, get_property, get_details, upgrade_ontap_image, update_svm_password, update_cvo_tags,
+                                           update_tier_level, update_instance_license_type, get_token):
+        data = self.set_default_args_pass_check()
+        data['svm_password'] = 'newpassword'
+        data['update_svm_password'] = True
+        data['ontap_version'] = 'ONTAP-9.10.1P3.T1.azure'
+        data['upgrade_ontap_version'] = True
+        data['instance_type'] = 'Standard_DS13_v2'
+        set_module_args(data)
 
-        modify = ['svm_password', 'azure_tag', 'tier_level']
+        modify = ['svm_password', 'azure_tag', 'tier_level', 'ontap_version', 'instance_type']
 
         my_cvo = {
-            'name': 'Dummyname',
+            'name': 'TestA',
             'publicId': 'test',
-            'svm_password': 'diffpassword',
-            'azure_tag': [{'tagKey': 'abc', 'tagValue': 'a124'}, {'tagKey': 'def', 'tagValue': 'b3424'}],
+            'svm_password': 'password',
+            'isHA': False,
+            'azure_tag': [{'tag_key': 'keya', 'tag_value': 'valuea'}, {'tag_key': 'keyb', 'tag_value': 'valueb'}],
         }
         get_cvo.return_value = my_cvo, None
 
-        cvo_property = {'name': 'Dummyname',
+        cvo_property = {'name': 'TestA',
                         'publicId': 'test',
-                        'ontapClusterProperties': {'capacityTierInfo': {'tierLevel': 'normal'}},
+                        'ontapClusterProperties': {
+                            'capacityTierInfo': {'tierLevel': 'normal'},
+                            'licensePackageName': 'Professional',
+                            'licenseType': {'capacityLimit': {'size': 2000.0, 'unit': 'TB'},
+                                            'name': 'Cloud Volumes ONTAP Capacity Based Charging'},
+                            'ontapVersion': '9.10.0.T1.azure',
+                            'upgradeVersions': [{'autoUpdateAllowed': False,
+                                                 'imageVersion': 'ONTAP-9.10.1P3',
+                                                 'lastModified': 1634467078000}],
+                            'writingSpeedState': 'NORMAL'},
                         'providerProperties': {
+                            'cloudProviderAccountId': 'CloudProviderAccount-abcdwxyz',
                             'regionName': 'westus',
+                            'instanceType': 'Standard_DS4_v2',
                             'resourceGroup': {
-                                'name': 'Dummyname-rg',
+                                'name': 'TestA-rg',
                                 'location': 'westus',
                                 'tags': {
                                     'DeployedByOccm': 'true'
@@ -353,9 +396,20 @@ class TestMyModule(unittest.TestCase):
                             'vnetCidr': '10.0.0.0/24',
                             'tags': {
                                 'DeployedByOccm': 'true'
-                            }}
+                            }},
+                        'tenantId': 'Tenant-abCdEfg1',
+                        'workingEnvironmentTyp': 'VSA'
                         }
         get_property.return_value = cvo_property, None
+        cvo_details = {'cloudProviderName': 'Azure',
+                       'isHA': False,
+                       'name': 'TestA',
+                       'ontapClusterProperties': None,
+                       'publicId': 'test',
+                       'status': None,
+                       'userTags': {'DeployedByOccm': 'true', 'key1': 'value1'},
+                       'workingEnvironmentType': 'VSA'}
+        get_details.return_value = cvo_details, None
         get_token.return_value = 'test', 'test'
         my_obj = my_module()
 
@@ -366,6 +420,10 @@ class TestMyModule(unittest.TestCase):
                 update_cvo_tags.return_value = True, None
             elif item == 'tier_level':
                 update_tier_level.return_value = True, None
+            elif item == 'ontap_version':
+                upgrade_ontap_image.return_value = True, None
+            elif item == 'instance_type':
+                update_instance_license_type.return_value = True, None
 
         with pytest.raises(AnsibleExitJson) as exc:
             my_obj.apply()
