@@ -47,8 +47,8 @@ try:
 except ImportError:
     ansible_version = 'unknown'
 
-COLLECTION_VERSION = "21.15.1"
-CLIENT_APP_VERSION = "%s/" + COLLECTION_VERSION
+COLLECTION_VERSION = "21.17.3"
+CLIENT_APP_VERSION = "%s/%s" % ("%s", COLLECTION_VERSION)
 IMPORT_EXCEPTION = None
 
 try:
@@ -83,6 +83,14 @@ POW2_BYTE_MAP = dict(
     # Here, 1 kb = 1024
     bytes=1,
     b=1,
+    k=1024,
+    m=1024 ** 2,
+    g=1024 ** 3,
+    t=1024 ** 4,
+    p=1024 ** 5,
+    e=1024 ** 6,
+    z=1024 ** 7,
+    y=1024 ** 8,
     kb=1024,
     mb=1024 ** 2,
     gb=1024 ** 3,
@@ -90,7 +98,7 @@ POW2_BYTE_MAP = dict(
     pb=1024 ** 5,
     eb=1024 ** 6,
     zb=1024 ** 7,
-    yb=1024 ** 8
+    yb=1024 ** 8,
 )
 
 ERROR_MSG = dict(
@@ -171,6 +179,8 @@ def get_feature(module, feature_name):
         # for SVM, whch protocols can be allowed
         svm_allowable_protocols_rest=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs'],
         svm_allowable_protocols_zapi=['cifs', 'fcp', 'iscsi', 'nvme', 'nfs', 'ndmp', 'http'],
+        warn_or_fail_on_fabricpool_backend_change='fail',
+        no_cserver_ems=False                    # when True, don't attempt to find cserver and don't send cserver EMS
     )
 
     if module.params['feature_flags'] is not None and feature_name in module.params['feature_flags']:
@@ -188,30 +198,25 @@ def create_sf_connection(module, port=None):
     if HAS_SF_SDK and hostname and username and password:
         try:
             return ElementFactory.create(hostname, username, password, port=port)
-        except Exception:
-            raise Exception("Unable to create SF connection")
+        except Exception as exc:
+            raise Exception("Unable to create SF connection: %s" % exc)
     module.fail_json(msg="the python SolidFire SDK module is required")
 
 
 def set_auth_method(module, username, password, cert_filepath, key_filepath):
     error = None
     if password is None and username is None:
-        if cert_filepath is None and key_filepath is not None:
-            error = 'Error: cannot have a key file without a cert file'
-        elif cert_filepath is None:
-            error = 'Error: ONTAP module requires username/password or SSL certificate file(s)'
-        elif key_filepath is None:
-            auth_method = 'single_cert'
+        if cert_filepath is None:
+            error = ('Error: cannot have a key file without a cert file' if key_filepath is not None
+                     else 'Error: ONTAP module requires username/password or SSL certificate file(s)')
         else:
-            auth_method = 'cert_key'
+            auth_method = 'single_cert' if key_filepath is None else 'cert_key'
     elif password is not None and username is not None:
         if cert_filepath is not None or key_filepath is not None:
             error = 'Error: cannot have both basic authentication (username/password) ' +\
                     'and certificate authentication (cert/key files)'
-        elif has_feature(module, 'classic_basic_authorization'):
-            auth_method = 'basic_auth'
         else:
-            auth_method = 'speedy_basic_auth'
+            auth_method = 'basic_auth' if has_feature(module, 'classic_basic_authorization') else 'speedy_basic_auth'
     else:
         error = 'Error: username and password have to be provided together'
         if cert_filepath is not None or key_filepath is not None:
@@ -305,6 +310,10 @@ def setup_na_ontap_zapi(module, vserver=None, wrap_zapi=False, host_options=None
         server = zapi.NaServer(hostname, username=username, password=password, trace=trace)
     if vserver:
         server.set_vserver(vserver)
+    if host_options.get('use_rest') == 'always':
+        note = '' if https else '  Note: https is set to false.'
+        module.warn("Using ZAPI for %s, ignoring 'use_rest: always'.%s" % (module._name, note))
+
     set_zapi_port_and_transport(server, https, port, validate_certs)
     server.set_api_version(major=1, minor=(version or 110))
     server.set_server_type('FILER')
@@ -338,6 +347,8 @@ def is_zapi_missing_vserver_error(message):
 
 
 def ems_log_event_cserver(source, server, module):
+    if has_feature(module, 'no_cserver_ems'):
+        return
     results = get_cserver(server)
     cserver = setup_na_ontap_zapi(module=module, vserver=results)
     ems_log_event(source, cserver)
@@ -487,27 +498,30 @@ if HAS_NETAPP_LIB:
             msg += '  More info: %s' % repr(exc)
             self.module.fail_json(msg=msg)
 
+        def sanitize_xml(self, response):
+            # some ONTAP CLI commands return BEL on error
+            new_response = response.replace(b'\x07\n', b'')
+            # And 9.1 uses \r\n rather than \n !
+            new_response = new_response.replace(b'\x07\r\n', b'')
+            # And 9.7 may send backspaces
+            for code_point in get_feature(self.module, 'sanitize_code_points'):
+                if bytes([8]) == b'\x08':   # python 3
+                    byte = bytes([code_point])
+                elif chr(8) == b'\x08':     # python 2
+                    byte = chr(code_point)
+                else:                       # very unlikely, noop
+                    byte = b'.'
+                new_response = new_response.replace(byte, b'.')
+            return new_response
+
         def _parse_response(self, response):
             ''' handling XML parsing exception '''
             try:
                 return super(OntapZAPICx, self)._parse_response(response)
             except zapi.etree.XMLSyntaxError as exc:
                 if has_feature(self.module, 'sanitize_xml'):
-                    # some ONTAP CLI commands return BEL on error
-                    new_response = response.replace(b'\x07\n', b'')
-                    # And 9.1 uses \r\n rather than \n !
-                    new_response = new_response.replace(b'\x07\r\n', b'')
-                    # And 9.7 may send backspaces
-                    for code_point in get_feature(self.module, 'sanitize_code_points'):
-                        if bytes([8]) == b'\x08':   # python 3
-                            byte = bytes([code_point])
-                        elif chr(8) == b'\x08':     # python 2
-                            byte = chr(code_point)
-                        else:                       # very unlikely, noop
-                            byte = b'.'
-                        new_response = new_response.replace(byte, b'.')
                     try:
-                        return super(OntapZAPICx, self)._parse_response(new_response)
+                        return super(OntapZAPICx, self)._parse_response(self.sanitize_xml(response))
                     except Exception:
                         # ignore a second exception, we'll report the first one
                         pass
@@ -587,7 +601,7 @@ class OntapRestAPI(object):
         self.timeout = timeout
         port = self.host_options['http_port']
         if port is None:
-            self.url = 'https://' + self.hostname + '/api/'
+            self.url = 'https://%s/api/' % self.hostname
         else:
             self.url = 'https://%s:%d/api/' % (self.hostname, port)
         self.is_rest_error = None
@@ -606,7 +620,7 @@ class OntapRestAPI(object):
             logging.basicConfig(filename=LOG_FILE, level=logging.DEBUG, format='%(asctime)s %(levelname)-8s %(message)s')
 
     def requires_ontap_9_6(self, module_name):
-        self.requires_ontap_version(module_name)
+        return self.requires_ontap_version(module_name)
 
     def requires_ontap_version(self, module_name, version='9.6'):
         suffix = " - %s" % self.is_rest_error if self.is_rest_error is not None else ""
@@ -634,7 +648,7 @@ class OntapRestAPI(object):
         status_code = self.get_ontap_version_using_rest()
         msgs = []
         if self.use_rest == 'never':
-            msgs.append('Error: REST is required for this module, found: "use_rest: %s"' % self.module.params.get('use_rest'))
+            msgs.append('Error: REST is required for this module, found: "use_rest: %s".' % self.use_rest)
         if self.is_rest_error:
             msgs.append('Error using REST for version, error: %s.' % self.is_rest_error)
         if status_code != 200:
@@ -666,15 +680,32 @@ class OntapRestAPI(object):
     def send_request(self, method, api, params, json=None, headers=None):
         ''' send http request and process reponse, including error conditions '''
         url = self.url + api
+
+        def get_auth_args():
+            if self.auth_method == 'single_cert':
+                kwargs = dict(cert=self.cert_filepath)
+            elif self.auth_method == 'cert_key':
+                kwargs = dict(cert=(self.cert_filepath, self.key_filepath))
+            elif self.auth_method in ('basic_auth', 'speedy_basic_auth'):
+                # with requests, there is no challenge, eg no 401.
+                kwargs = dict(auth=(self.username, self.password))
+            else:
+                raise KeyError(self.auth_method)
+            return kwargs
+
+        status_code, json_dict, error_details = self._send_request(method, url, params, json, headers, get_auth_args())
+
+        return status_code, json_dict, error_details
+
+    def _send_request(self, method, url, params, json, headers, auth_args):
         status_code = None
-        content = None
         json_dict = None
         json_error = None
         error_details = None
         if headers is None:
             headers = self.build_headers()
 
-        def check_contents(response):
+        def fail_on_non_empty_value(response):
             '''json() may fail on an empty value, but it's OK if no response is expected.
                To avoid false positives, only report an issue when we expect to read a value.
                The first get will see it.
@@ -689,27 +720,16 @@ class OntapRestAPI(object):
             try:
                 json = response.json()
             except ValueError:
-                check_contents(response)
+                fail_on_non_empty_value(response)
                 return None, None
-            error = json.get('error')
-            return json, error
-
-        if self.auth_method == 'single_cert':
-            kwargs = dict(cert=self.cert_filepath)
-        elif self.auth_method == 'cert_key':
-            kwargs = dict(cert=(self.cert_filepath, self.key_filepath))
-        elif self.auth_method in ('basic_auth', 'speedy_basic_auth'):
-            # with requests, there is no challenge, eg no 401.
-            kwargs = dict(auth=(self.username, self.password))
-        else:
-            raise KeyError(self.auth_method)
+            return json, json.get('error')
 
         self.log_debug('sending', repr(dict(method=method, url=url, verify=self.verify, params=params,
-                                            timeout=self.timeout, json=json, headers=headers, **kwargs)))
+                                            timeout=self.timeout, json=json, headers=headers, auth_args=auth_args)))
         try:
             response = requests.request(method, url, verify=self.verify, params=params,
-                                        timeout=self.timeout, json=json, headers=headers, **kwargs)
-            content = response.content  # for debug purposes
+                                        timeout=self.timeout, json=json, headers=headers, **auth_args)
+            self.log_debug(status_code, response.content)
             status_code = response.status_code
             # If the response was successful, no Exception will be raised
             response.raise_for_status()
@@ -717,7 +737,7 @@ class OntapRestAPI(object):
         except requests.exceptions.HTTPError as err:
             try:
                 __, json_error = get_json(response)
-            except ValueError:
+            except (AttributeError, ValueError):
                 json_error = None
             if json_error is None:
                 self.log_error(status_code, 'HTTP error: %s' % err)
@@ -733,10 +753,9 @@ class OntapRestAPI(object):
         if json_error is not None:
             self.log_error(status_code, 'Endpoint error: %d: %s' % (status_code, json_error))
             error_details = json_error
-        self.log_debug(status_code, content)
-        if not json_dict and method == 'OPTIONS':
+        if not error_details and not json_dict and method == 'OPTIONS':
             # OPTIONS provides the list of supported verbs
-            json_dict['Allow'] = response.headers['Allow']
+            json_dict['Allow'] = response.headers.get('Allow')
         return status_code, json_dict, error_details
 
     def wait_on_job(self, job, timeout=600, increment=60):
@@ -754,7 +773,6 @@ class OntapRestAPI(object):
         #         }
         #     }
         # }
-        keep_running = True
         error = None
         message = None
         runtime = 0
@@ -847,11 +865,9 @@ class OntapRestAPI(object):
                 self.ontap_version[key] = version.get(key, -1)
             except AttributeError:
                 self.ontap_version[key] = -1
-        self.ontap_version['valid'] = True
-        for key in self.ontap_version:
-            if self.ontap_version[key] == -1:
-                self.ontap_version['valid'] = False
-                break
+        self.ontap_version['valid'] = all(
+            self.ontap_version[key] != -1 for key in self.ontap_version if key != 'valid'
+        )
 
     def get_ontap_version(self):
         if self.ontap_version['valid']:
@@ -915,16 +931,25 @@ class OntapRestAPI(object):
         # we're now using 'auto'
         if used_unsupported_rest_properties:
             # force ZAPI if some parameter requires it
-            if self.get_ontap_version()[0:2] > (9, 5):
+            if self.get_ontap_version()[:2] > (9, 5):
                 self.module.warn('Falling back to ZAPI because of unsupported option(s) or option value(s) in REST: %s' % used_unsupported_rest_properties)
             return False, None
-        if self.get_ontap_version()[0:2] in ((9, 4), (9, 5)):
+        if self.get_ontap_version()[:2] in ((9, 4), (9, 5)):
             # we can't trust REST support on 9.5, and not at all on 9.4
             return False, None
         if status_code == 200:
             return True, None
         return False, None
 
+    def is_rest_supported_properties(self, parameters, unsupported_rest_properties=None, partially_supported_rest_properties=None):
+        used_unsupported_rest_properties = None
+        if unsupported_rest_properties:
+            used_unsupported_rest_properties = [x for x in unsupported_rest_properties if x in parameters]
+        use_rest, error = self.is_rest(used_unsupported_rest_properties, partially_supported_rest_properties, parameters)
+        if error:
+            self.module.fail_json(msg=error)
+        return use_rest
+
     def is_rest(self, used_unsupported_rest_properties=None, partially_supported_rest_properties=None, parameters=None):
         ''' only return error if there is a reason to '''
         use_rest, error = self._is_rest(used_unsupported_rest_properties, partially_supported_rest_properties, parameters)
