@@ -7,14 +7,13 @@ from __future__ import (absolute_import, division, print_function)
 
 __metaclass__ = type
 
-import json
 import pytest
 
-from ansible.module_utils import basic
-from ansible.module_utils._text import to_bytes
 from ansible_collections.netapp.ontap.tests.unit.compat import unittest
 from ansible_collections.netapp.ontap.tests.unit.compat.mock import patch, Mock
 import ansible_collections.netapp.ontap.plugins.module_utils.netapp as netapp_utils
+from ansible_collections.netapp.ontap.tests.unit.plugins.module_utils.ansible_mocks import set_module_args,\
+    AnsibleFailJson, AnsibleExitJson, patch_ansible, assert_warning_was_raised, print_warnings
 
 from ansible_collections.netapp.ontap.plugins.modules.na_ontap_volume \
     import NetAppOntapVolume as volume_module  # module under test
@@ -62,12 +61,14 @@ SRR = {
                        "state": "online",
                        "style": "flexvol",
                        "tiering": {
-                           "policy": "backup"
+                           "policy": "backup",
+                           "min_cooling_days": 0
                        },
                        "type": "rw",
                        "aggregates": [
                            {
-                               "name": "aggr1"
+                               "name": "aggr1",
+                               "uuid": "aggr1_uuid"
                            }
                        ],
                        "encryption": {
@@ -94,6 +95,10 @@ SRR = {
                            "uuid": "0a42a3d9-0c29-11ec-a267-005056b30cfa"
                        },
                        "space": {
+                           "logical_space": {
+                               "enforcement": False,
+                               "reporting": False,
+                           },
                            "size": 10737418240,
                            "snapshot": {
                                "reserve_percent": 5
@@ -207,46 +212,18 @@ SRR = {
                                      "name": "test_app",
                                      "nas": {
                                          "application_components": [{'xxx': 1}]}
-                                     }]}, None)
+                                     }]}, None),
+    'get_aggr_one_object_store': (200,
+                                  {'records': ['one']}, None),
+    'get_aggr_two_object_stores': (200,
+                                   {'records': ['two']}, None),
 }
 
 
-def set_module_args(args):
-    """prepare arguments so that they will be picked up during module creation"""
-    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
-    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access
-
-
-class AnsibleExitJson(Exception):
-    """Exception class to be raised by module.exit_json and caught by the test case"""
-
-
-class AnsibleFailJson(Exception):
-    """Exception class to be raised by module.fail_json and caught by the test case"""
-
-
-def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
-    """function to patch over exit_json; package return data into an exception"""
-    if 'changed' not in kwargs:
-        kwargs['changed'] = False
-    raise AnsibleExitJson(kwargs)
-
-
-def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
-    """function to patch over fail_json; package return data into an exception"""
-    kwargs['failed'] = True
-    raise AnsibleFailJson(kwargs)
-
-
 class TestMyModule(unittest.TestCase):
     ''' a group of related Unit Tests '''
 
     def setUp(self):
-        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
-                                                 exit_json=exit_json,
-                                                 fail_json=fail_json)
-        self.mock_module_helper.start()
-        self.addCleanup(self.mock_module_helper.stop)
         # self.server = MockONTAPConnection()
         self.mock_vserver = {
             'name': 'test_svm',
@@ -567,7 +544,7 @@ class TestMyModule(unittest.TestCase):
         ]
         with pytest.raises(AnsibleFailJson) as exc:
             self.get_volume_mock_object().apply()
-        msg = "Error unmounting volume test_svm: calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error."
+        msg = 'Error unmounting volume test_svm: with path "", calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error.'
         assert exc.value.args[0]['msg'] == msg
 
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
@@ -585,6 +562,21 @@ class TestMyModule(unittest.TestCase):
             self.get_volume_mock_object().apply()
         assert exc.value.args[0]['changed']
 
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_successfully_volume_mount_do_nothing_rest(self, mock_request):
+        data = dict(self.mock_args_volume())
+        data['junction_path'] = ""
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['get_volume_mount'],  # Get Volume
+            SRR['no_record'],  # Mount Volume
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleExitJson) as exc:
+            self.get_volume_mock_object().apply()
+        assert not exc.value.args[0]['changed']
+
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
     def test_rest_error_volume_mount_rest(self, mock_request):
         data = dict(self.mock_args_volume())
@@ -643,6 +635,9 @@ class TestMyModule(unittest.TestCase):
         data['user_id'] = 5
         data['volume_security_style'] = 'mixed'
         data['comment'] = 'carchi8py was here'
+        data['tiering_minimum_cooling_days'] = 10
+        data['logical_space_enforcement'] = True
+        data['logical_space_reporting'] = True
         set_module_args(data)
         mock_request.side_effect = [
             SRR['is_rest'],
@@ -737,12 +732,14 @@ class TestMyModule(unittest.TestCase):
         data['encrypt'] = True
         data['junction_path'] = '/this/path'
         data['tiering_policy'] = 'backup'
+        data['tiering_minimum_cooling_days'] = 10
         set_module_args(data)
         mock_request.side_effect = [
             SRR['is_rest'],
             SRR['no_record'],  # Get Volume
             SRR['no_record'],  # Create Volume
             SRR['get_volume'],
+            SRR['no_record'],  # modify Volume
             SRR['end_of_sequence']
         ]
         with pytest.raises(AnsibleExitJson) as exc:
@@ -1016,7 +1013,7 @@ class TestMyModule(unittest.TestCase):
         ]
         with pytest.raises(AnsibleFailJson) as exc:
             self.get_volume_mock_object().apply()
-        msg = "Error set efficiency for volume test_svm: calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error."
+        msg = "Error setting efficiency for volume test_svm: calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error."
         assert exc.value.args[0]['msg'] == msg
 
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
@@ -1033,7 +1030,7 @@ class TestMyModule(unittest.TestCase):
         ]
         with pytest.raises(AnsibleFailJson) as exc:
             self.get_volume_mock_object().apply()
-        msg = "Error set efficiency for volume test_svm: calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error."
+        msg = "Error setting efficiency for volume test_svm: calling: storage/volumes/7882901a-1aef-11ec-a267-005056b30cfa: got Expected error."
         assert exc.value.args[0]['msg'] == msg
 
     @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
@@ -1048,3 +1045,130 @@ class TestMyModule(unittest.TestCase):
             self.get_volume_mock_object().apply()
         msg = "Minimum version of ONTAP for efficiency_policy is (9, 7)\n"
         assert exc.value.args[0]['msg'] == msg
+
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_error_modify_volume_tiering_minimum_cooling_days_98(self, mock_request):
+        data = dict(self.mock_args_volume())
+        data['tiering_minimum_cooling_days'] = 2
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest_96'],
+        ]
+        with pytest.raises(AnsibleFailJson) as exc:
+            self.get_volume_mock_object().apply()
+        msg = "Minimum version of ONTAP for tiering_minimum_cooling_days is (9, 8)\n"
+        assert exc.value.args[0]['msg'] == msg
+
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_successfully_created_with_logical_space(self, mock_request):
+        data = dict(self.mock_args_volume())
+        data['logical_space_enforcement'] = False
+        data['logical_space_reporting'] = False
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['no_record'],  # Get Volume
+            SRR['no_record'],  # Create Volume
+            SRR['get_volume'],
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleExitJson) as exc:
+            self.get_volume_mock_object().apply()
+        assert exc.value.args[0]['changed']
+
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_error_modify_backend_fabricpool(self, mock_request):
+        data = dict(self.mock_args())
+        data['nas_application_template']['tiering'] = {'control': 'required'}
+        data['feature_flags'] = {'warn_or_fail_on_fabricpool_backend_change': 'fail'}
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['get_volume'],
+            SRR['no_record'],       # get_aggr_object_stores
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleFailJson) as exc:
+            self.get_volume_mock_object().apply()
+        msg = "Error: changing a volume from one backend to another is not allowed.  Current tiering control: disallowed, desired: required."
+        assert exc.value.args[0]['msg'] == msg
+
+        data['feature_flags'] = {'warn_or_fail_on_fabricpool_backend_change': 'invalid'}
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['get_volume'],
+            SRR['no_record'],   # modify
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleExitJson) as exc:
+            self.get_volume_mock_object().apply()
+        print(exc.value.args[0])
+        print_warnings()
+        warning = "Unexpected value 'invalid' for warn_or_fail_on_fabricpool_backend_change, expecting: None, 'ignore', 'fail', 'warn'"
+        assert_warning_was_raised(warning)
+
+        data['feature_flags'] = {'warn_or_fail_on_fabricpool_backend_change': 'warn'}
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['get_volume'],
+            SRR['no_record'],   # get_aggr_object_stores
+            SRR['no_record'],   # modify
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleExitJson) as exc:
+            self.get_volume_mock_object().apply()
+        warning = "Ignored " + msg
+        print_warnings()
+        assert_warning_was_raised(warning)
+
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_negative_modify_backend_fabricpool(self, mock_request):
+        ''' fail to get aggregate object store'''
+        data = dict(self.mock_args())
+        data['nas_application_template']['tiering'] = {'control': 'required'}
+        data['feature_flags'] = {'warn_or_fail_on_fabricpool_backend_change': 'fail'}
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['get_volume'],
+            SRR['generic_error'],
+            SRR['end_of_sequence']
+        ]
+        with pytest.raises(AnsibleFailJson) as exc:
+            self.get_volume_mock_object().apply()
+        msg = "Error getting object store for aggregate: aggr1: calling: storage/aggregates/aggr1_uuid/cloud-stores: got Expected error."
+        assert exc.value.args[0]['msg'] == msg
+
+    @patch('ansible_collections.netapp.ontap.plugins.module_utils.netapp.OntapRestAPI.send_request')
+    def test_rest_tiering_control(self, mock_request):
+        ''' The volume is supported by one or more aggregates
+            If all aggregates are associated with one or more object stores, the volume has a FabricPool backend.
+            If all aggregates are not associated with one or more object stores, the volume meets the 'disallowed' criteria.
+        '''
+        data = dict(self.mock_args())
+        data['nas_application_template']['tiering'] = {'control': 'required'}
+        data['feature_flags'] = {'warn_or_fail_on_fabricpool_backend_change': 'fail'}
+        set_module_args(data)
+        mock_request.side_effect = [
+            SRR['is_rest'],
+            SRR['no_record'],                   # get_aggr_object_stores aggr1
+            SRR['no_record'],                   # get_aggr_object_stores aggr2
+            SRR['get_aggr_one_object_store'],   # get_aggr_object_stores aggr1
+            SRR['no_record'],                   # get_aggr_object_stores aggr2
+            SRR['get_aggr_two_object_stores'],  # get_aggr_object_stores aggr1
+            SRR['get_aggr_one_object_store'],   # get_aggr_object_stores aggr2
+            SRR['end_of_sequence']
+        ]
+        current = {'aggregates': [{'name': 'aggr1', 'uuid': 'uuid1'}, {'name': 'aggr2', 'uuid': 'uuid2'}]}
+        vol_object = self.get_volume_mock_object()
+        result = vol_object.tiering_control(current)
+        assert result == 'disallowed'
+        result = vol_object.tiering_control(current)
+        assert result == 'best_effort'
+        result = vol_object.tiering_control(current)
+        assert result == 'required'
+        current = {'aggregates': []}
+        result = vol_object.tiering_control(current)
+        assert result is None
