@@ -34,7 +34,7 @@ options:
       - When supplied, this argument will define the information to be collected.
         Possible values for this include all, minimum, config, performance,
         capacity, network, subnets, lags, filesystems, snapshots, buckets,
-        replication, policies, arrays and admins.
+        replication, policies, arrays, accounts, admins, ad and kerberos.
     required: false
     type: list
     elements: str
@@ -94,6 +94,41 @@ purefb_info:
                 "public_key": null
             },
         },
+        "buckets": {
+            "central": {
+                "account_name": "jake",
+                "created": 1628900154000,
+                "data_reduction": null,
+                "destroyed": false,
+                "id": "43758f09-9e71-7bf7-5757-2028a95a2b65",
+                "lifecycle_rules": {},
+                "object_count": 0,
+                "snapshot_space": 0,
+                "time_remaining": null,
+                "total_physical_space": 0,
+                "unique_space": 0,
+                "versioning": "none",
+                "virtual_space": 0
+            },
+            "test": {
+                "account_name": "acme",
+                "created": 1630591952000,
+                "data_reduction": 3.6,
+                "destroyed": false,
+                "id": "d5f6149c-fbef-f3c5-58b6-8fd143110ba9",
+                "lifecycle_rules": {
+                    "test": {
+                        "abort_incomplete_multipart_uploads_after (days)": 1,
+                        "cleanup_expired_object_delete_marker": true,
+                        "enabled": true,
+                        "keep_current_version_for (days)": null,
+                        "keep_current_version_until": "2023-12-21",
+                        "keep_previous_version_for (days)": null,
+                        "prefix": "foo"
+                    }
+                },
+            },
+        },
         "capacity": {
             "aggregate": {
                 "data_reduction": 1.1179228,
@@ -222,7 +257,8 @@ purefb_info:
             "object_store_users": 1,
             "purity_version": "2.2.0",
             "snapshots": 1,
-            "total_capacity": 83359896948925
+            "total_capacity": 83359896948925,
+            "smb_mode": "native"
         },
         "filesystems": {
             "k8s-pvc-d24b1357-579e-11e8-811f-ecf4bbc88f54": {
@@ -395,8 +431,10 @@ purefb_info:
 from ansible.module_utils.basic import AnsibleModule
 from ansible_collections.purestorage.flashblade.plugins.module_utils.purefb import (
     get_blade,
+    get_system,
     purefb_argument_spec,
 )
+from datetime import datetime
 
 
 MIN_REQUIRED_API_VERSION = "1.3"
@@ -405,9 +443,32 @@ POLICIES_API_VERSION = "1.5"
 CERT_GROUPS_API_VERSION = "1.8"
 REPLICATION_API_VERSION = "1.9"
 MULTIPROTOCOL_API_VERSION = "1.11"
+MIN_32_API = "2.0"
+LIFECYCLE_API_VERSION = "2.1"
+SMB_MODE_API_VERSION = "2.2"
+NFS_POLICY_API_VERSION = "2.3"
 
 
-def generate_default_dict(blade):
+def _millisecs_to_time(millisecs):
+    if millisecs:
+        return (str(int(millisecs / 3600000 % 24)).zfill(2) + ":00",)
+    return None
+
+
+def _bytes_to_human(bytes_number):
+    if bytes_number:
+        labels = ["B/s", "KB/s", "MB/s", "GB/s", "TB/s", "PB/s"]
+        i = 0
+        double_bytes = bytes_number
+        while i < len(labels) and bytes_number >= 1024:
+            double_bytes = bytes_number / 1024.0
+            i += 1
+            bytes_number = bytes_number / 1024
+        return str(round(double_bytes, 2)) + " " + labels[i]
+    return None
+
+
+def generate_default_dict(module, blade):
     default_info = {}
     defaults = blade.arrays.list_arrays().items[0]
     default_info["flashblade_name"] = defaults.name
@@ -448,6 +509,30 @@ def generate_default_dict(blade):
         )
         default_info["targets"] = len(blade.targets.list_targets().items)
         default_info["kerberos_keytabs"] = len(blade.keytabs.list_keytabs().items)
+    # This section is just for REST 2.x features
+    if MIN_32_API in api_version:
+        blade = get_system(module)
+        blade_info = list(blade.get_arrays().items)[0]
+        default_info["object_store_virtual_hosts"] = len(
+            blade.get_object_store_virtual_hosts().items
+        )
+        default_info["api_clients"] = len(blade.get_api_clients().items)
+        default_info["idle_timeout"] = int(blade_info.idle_timeout / 60000)
+        if list(blade.get_arrays_eula().items)[0].signature.accepted:
+            default_info["EULA"] = "Signed"
+        else:
+            default_info["EULA"] = "Not Signed"
+        if NFS_POLICY_API_VERSION in api_version:
+            admin_settings = list(blade.get_admins_settings().items)[0]
+            default_info["max_login_attempts"] = admin_settings.max_login_attempts
+            default_info["min_password_length"] = admin_settings.min_password_length
+            if admin_settings.lockout_duration:
+                default_info["lockout_duration"] = (
+                    str(admin_settings.lockout_duration / 1000) + " seconds"
+                )
+        if NFS_POLICY_API_VERSION in api_version:
+            default_info["smb_mode"] = blade_info.smb_mode
+
     return default_info
 
 
@@ -664,7 +749,7 @@ def generate_lag_dict(blade):
     return lag_info
 
 
-def generate_admin_dict(blade):
+def generate_admin_dict(module, blade):
     admin_info = {}
     api_version = blade.api_version.list_versions().versions
     if MULTIPROTOCOL_API_VERSION in api_version:
@@ -676,6 +761,27 @@ def generate_admin_dict(blade):
                 "public_key": admins.items[admin].public_key,
                 "local": admins.items[admin].is_local,
             }
+
+    if MIN_32_API in api_version:
+        bladev2 = get_system(module)
+        admins = list(bladev2.get_admins().items)
+        for admin in range(0, len(admins)):
+            admin_name = admins[admin].name
+            if admins[admin].api_token.expires_at:
+                admin_info[admin_name]["token_expires"] = datetime.fromtimestamp(
+                    admins[admin].api_token.expires_at / 1000
+                ).strftime("%Y-%m-%d %H:%M:%S")
+            else:
+                admin_info[admin_name]["token_expires"] = None
+            admin_info[admin_name]["token_created"] = datetime.fromtimestamp(
+                admins[admin].api_token.created_at / 1000
+            ).strftime("%Y-%m-%d %H:%M:%S")
+            admin_info[admin_name]["role"] = admins[admin].role.name
+            if NFS_POLICY_API_VERSION in api_version:
+                admin_info[admin_name]["locked"] = admins[admin].locked
+                admin_info[admin_name]["lockout_remaining"] = admins[
+                    admin
+                ].lockout_remaining
     return admin_info
 
 
@@ -728,7 +834,7 @@ def generate_file_repl_dict(blade):
     return file_repl_info
 
 
-def generate_bucket_repl_dict(blade):
+def generate_bucket_repl_dict(module, blade):
     bucket_repl_info = {}
     bucket_links = blade.bucket_replica_links.list_bucket_replica_links()
     for linkcnt in range(0, len(bucket_links.items)):
@@ -741,7 +847,25 @@ def generate_bucket_repl_dict(blade):
             "remote_bucket": bucket_links.items[linkcnt].remote_bucket.name,
             "remote_credentials": bucket_links.items[linkcnt].remote_credentials.name,
             "recovery_point": bucket_links.items[linkcnt].recovery_point,
+            "object_backlog": {},
         }
+    api_version = blade.api_version.list_versions().versions
+    if SMB_MODE_API_VERSION in api_version:
+        blade = get_system(module)
+        bucket_links = list(blade.get_bucket_replica_links().items)
+        for linkcnt in range(0, len(bucket_links)):
+            bucket_name = bucket_links[linkcnt].local_bucket.name
+            bucket_repl_info[bucket_name]["object_backlog"] = {
+                "bytes_count": bucket_links[linkcnt].object_backlog.bytes_count,
+                "delete_ops_count": bucket_links[
+                    linkcnt
+                ].object_backlog.delete_ops_count,
+                "other_ops_count": bucket_links[linkcnt].object_backlog.other_ops_count,
+                "put_ops_count": bucket_links[linkcnt].object_backlog.put_ops_count,
+            }
+            bucket_repl_info[bucket_name]["cascading_enabled"] = bucket_links[
+                linkcnt
+            ].cascading_enabled
     return bucket_repl_info
 
 
@@ -837,23 +961,44 @@ def generate_snap_transfer_dict(blade):
     return snap_transfer_info
 
 
-def generate_array_conn_dict(blade):
+def generate_array_conn_dict(module, blade):
     array_conn_info = {}
+    arraysv2 = {}
+    api_version = blade.api_version.list_versions().versions
     arrays = blade.array_connections.list_array_connections()
+    if NFS_POLICY_API_VERSION in api_version:
+        bladev2 = get_system(module)
+        arraysv2 = list(bladev2.get_array_connections().items)
     for arraycnt in range(0, len(arrays.items)):
         array = arrays.items[arraycnt].remote.name
         array_conn_info[array] = {
             "encrypted": arrays.items[arraycnt].encrypted,
             "replication_addresses": arrays.items[arraycnt].replication_addresses,
             "management_address": arrays.items[arraycnt].management_address,
-            "id": arrays.items[arraycnt].remote.id,
             "status": arrays.items[arraycnt].status,
             "version": arrays.items[arraycnt].version,
+            "throttle": [],
         }
         if arrays.items[arraycnt].encrypted:
             array_conn_info[array]["ca_certificate_group"] = arrays.items[
                 arraycnt
             ].ca_certificate_group.name
+        for v2array in range(0, len(arraysv2)):
+            if arraysv2[v2array].remote.name == array:
+                array_conn_info[array]["throttle"] = {
+                    "default_limit": _bytes_to_human(
+                        arraysv2[v2array].throttle.default_limit
+                    ),
+                    "window_limit": _bytes_to_human(
+                        arraysv2[v2array].throttle.window_limit
+                    ),
+                    "window_start": _millisecs_to_time(
+                        arraysv2[v2array].throttle.window.start
+                    ),
+                    "window_end": _millisecs_to_time(
+                        arraysv2[v2array].throttle.window.end
+                    ),
+                }
     return array_conn_info
 
 
@@ -871,7 +1016,7 @@ def generate_policies_dict(blade):
     return policies_info
 
 
-def generate_bucket_dict(blade):
+def generate_bucket_dict(module, blade):
     bucket_info = {}
     buckets = blade.buckets.list_buckets()
     for bckt in range(0, len(buckets.items)):
@@ -889,11 +1034,211 @@ def generate_bucket_dict(blade):
             "created": buckets.items[bckt].created,
             "destroyed": buckets.items[bckt].destroyed,
             "time_remaining": buckets.items[bckt].time_remaining,
+            "lifecycle_rules": {},
         }
+    api_version = blade.api_version.list_versions().versions
+    if LIFECYCLE_API_VERSION in api_version:
+        blade = get_system(module)
+        all_rules = list(blade.get_lifecycle_rules().items)
+        for rule in range(0, len(all_rules)):
+            bucket_name = all_rules[rule].bucket.name
+            rule_id = all_rules[rule].rule_id
+            if all_rules[rule].keep_previous_version_for:
+                keep_previous_version_for = int(
+                    all_rules[rule].keep_previous_version_for / 86400000
+                )
+            else:
+                keep_previous_version_for = None
+            if all_rules[rule].keep_current_version_for:
+                keep_current_version_for = int(
+                    all_rules[rule].keep_current_version_for / 86400000
+                )
+            else:
+                keep_current_version_for = None
+            if all_rules[rule].abort_incomplete_multipart_uploads_after:
+                abort_incomplete_multipart_uploads_after = int(
+                    all_rules[rule].abort_incomplete_multipart_uploads_after / 86400000
+                )
+            else:
+                abort_incomplete_multipart_uploads_after = None
+            if all_rules[rule].keep_current_version_until:
+                keep_current_version_until = datetime.fromtimestamp(
+                    all_rules[rule].keep_current_version_until / 1000
+                ).strftime("%Y-%m-%d")
+            else:
+                keep_current_version_until = None
+            bucket_info[bucket_name]["lifecycle_rules"][rule_id] = {
+                "keep_previous_version_for (days)": keep_previous_version_for,
+                "keep_current_version_for (days)": keep_current_version_for,
+                "keep_current_version_until": keep_current_version_until,
+                "prefix": all_rules[rule].prefix,
+                "enabled": all_rules[rule].enabled,
+                "abort_incomplete_multipart_uploads_after (days)": abort_incomplete_multipart_uploads_after,
+                "cleanup_expired_object_delete_marker": all_rules[
+                    rule
+                ].cleanup_expired_object_delete_marker,
+            }
     return bucket_info
 
 
-def generate_fs_dict(blade):
+def generate_kerb_dict(blade):
+    kerb_info = {}
+    keytabs = list(blade.get_keytabs().items)
+    for ktab in range(0, len(keytabs)):
+        keytab_name = keytabs[ktab].prefix
+        kerb_info[keytab_name] = {}
+        for key in range(0, len(keytabs)):
+            if keytabs[key].prefix == keytab_name:
+                kerb_info[keytab_name][keytabs[key].suffix] = {
+                    "fqdn": keytabs[key].fqdn,
+                    "kvno": keytabs[key].kvno,
+                    "principal": keytabs[key].principal,
+                    "realm": keytabs[key].realm,
+                    "encryption_type": keytabs[key].encryption_type,
+                }
+    return kerb_info
+
+
+def generate_ad_dict(blade):
+    ad_info = {}
+    active_directory = blade.get_active_directory()
+    if active_directory.total_item_count != 0:
+        ad_account = list(active_directory.items)[0]
+        ad_info[ad_account.name] = {
+            "computer": ad_account.computer_name,
+            "domain": ad_account.domain,
+            "directory_servers": ad_account.directory_servers,
+            "kerberos_servers": ad_account.kerberos_servers,
+            "service_principals": ad_account.service_principal_names,
+            "join_ou": ad_account.join_ou,
+            "encryption_types": ad_account.encryption_types,
+        }
+    return ad_info
+
+
+def generate_object_store_access_policies_dict(blade):
+    policies_info = {}
+    policies = list(blade.get_object_store_access_policies().items)
+    for policy in range(0, len(policies)):
+        policy_name = policies[policy].name
+        policies_info[policy_name] = {
+            "ARN": policies[policy].arn,
+            "description": policies[policy].description,
+            "enabled": policies[policy].enabled,
+            "local": policies[policy].is_local,
+            "rules": [],
+        }
+        for rule in range(0, len(policies[policy].rules)):
+            policies_info[policy_name]["rules"].append(
+                {
+                    "actions": policies[policy].rules[rule].actions,
+                    "conditions": {
+                        "source_ips": policies[policy]
+                        .rules[rule]
+                        .conditions.source_ips,
+                        "s3_delimiters": policies[policy]
+                        .rules[rule]
+                        .conditions.s3_delimiters,
+                        "s3_prefixes": policies[policy]
+                        .rules[rule]
+                        .conditions.s3_prefixes,
+                    },
+                    "effect": policies[policy].rules[rule].effect,
+                    "name": policies[policy].rules[rule].name,
+                }
+            )
+    return policies_info
+
+
+def generate_nfs_export_policies_dict(blade):
+    policies_info = {}
+    policies = list(blade.get_nfs_export_policies().items)
+    for policy in range(0, len(policies)):
+        policy_name = policies[policy].name
+        policies_info[policy_name] = {
+            "local": policies[policy].is_local,
+            "enabled": policies[policy].enabled,
+            "rules": [],
+        }
+        for rule in range(0, len(policies[policy].rules)):
+            policies_info[policy_name]["rules"].append(
+                {
+                    "access": policies[policy].rules[rule].access,
+                    "anongid": policies[policy].rules[rule].anongid,
+                    "anonuid": policies[policy].rules[rule].anonuid,
+                    "atime": policies[policy].rules[rule].atime,
+                    "client": policies[policy].rules[rule].client,
+                    "fileid_32bit": policies[policy].rules[rule].fileid_32bit,
+                    "permission": policies[policy].rules[rule].permission,
+                    "secure": policies[policy].rules[rule].secure,
+                    "security": policies[policy].rules[rule].security,
+                    "index": policies[policy].rules[rule].index,
+                }
+            )
+    return policies_info
+
+
+def generate_object_store_accounts_dict(blade):
+    account_info = {}
+    accounts = list(blade.get_object_store_accounts().items)
+    for account in range(0, len(accounts)):
+        acc_name = accounts[account].name
+        account_info[acc_name] = {
+            "object_count": accounts[account].object_count,
+            "data_reduction": accounts[account].space.data_reduction,
+            "snapshots_space": accounts[account].space.snapshots,
+            "total_physical_space": accounts[account].space.total_physical,
+            "unique_space": accounts[account].space.unique,
+            "virtual_space": accounts[account].space.virtual,
+            "users": {},
+        }
+        acc_users = list(
+            blade.get_object_store_users(filter='name="' + acc_name + '/*"').items
+        )
+        for acc_user in range(0, len(acc_users)):
+            user_name = acc_users[acc_user].name.split("/")[1]
+            account_info[acc_name]["users"][user_name] = {"keys": [], "policies": []}
+            if (
+                blade.get_object_store_access_keys(
+                    filter='user.name="' + acc_users[acc_user].name + '"'
+                ).total_item_count
+                != 0
+            ):
+                access_keys = list(
+                    blade.get_object_store_access_keys(
+                        filter='user.name="' + acc_users[acc_user].name + '"'
+                    ).items
+                )
+                for key in range(0, len(access_keys)):
+                    account_info[acc_name]["users"][user_name]["keys"].append(
+                        {
+                            "name": access_keys[key].name,
+                            "enabled": bool(access_keys[key].enabled),
+                        }
+                    )
+            if (
+                blade.get_object_store_access_policies_object_store_users(
+                    member_names=[acc_users[acc_user].name]
+                ).total_item_count
+                != 0
+            ):
+                policies = list(
+                    blade.get_object_store_access_policies_object_store_users(
+                        member_names=[acc_users[acc_user].name]
+                    ).items
+                )
+                for policy in range(0, len(policies)):
+                    account_info[acc_name]["users"][user_name]["policies"].append(
+                        policies[policy].policy.name
+                    )
+    return account_info
+
+
+def generate_fs_dict(module, blade):
+    api_version = blade.api_version.list_versions().versions
+    if NFS_POLICY_API_VERSION in api_version:
+        bladev2 = get_system(module)
+        fsys_v2 = list(bladev2.get_file_systems().items)
     fs_info = {}
     fsys = blade.file_systems.list_file_systems()
     for fsystem in range(0, len(fsys.items)):
@@ -903,13 +1248,14 @@ def generate_fs_dict(blade):
             "snapshot_enabled": fsys.items[fsystem].snapshot_directory_enabled,
             "provisioned": fsys.items[fsystem].provisioned,
             "destroyed": fsys.items[fsystem].destroyed,
+            "nfs_rules": fsys.items[fsystem].nfs.rules,
+            "nfs_v3": getattr(fsys.items[fsystem].nfs, "v3_enabled", False),
+            "nfs_v4_1": getattr(fsys.items[fsystem].nfs, "v4_1_enabled", False),
         }
         if fsys.items[fsystem].http.enabled:
             fs_info[share]["http"] = fsys.items[fsystem].http.enabled
         if fsys.items[fsystem].smb.enabled:
             fs_info[share]["smb_mode"] = fsys.items[fsystem].smb.acl_mode
-        if fsys.items[fsystem].nfs.enabled:
-            fs_info[share]["nfs_rules"] = fsys.items[fsystem].nfs.rules
         api_version = blade.api_version.list_versions().versions
         if MULTIPROTOCOL_API_VERSION in api_version:
             fs_info[share]["multi_protocol"] = {
@@ -930,6 +1276,12 @@ def generate_fs_dict(blade):
                 "is_local": fsys.items[fsystem].source.is_local,
                 "name": fsys.items[fsystem].source.name,
             }
+            if NFS_POLICY_API_VERSION in api_version:
+                for v2fs in range(0, len(fsys_v2)):
+                    if fsys_v2[v2fs].name == share:
+                        fs_info[share]["export_policy"] = fsys_v2[
+                            v2fs
+                        ].nfs.export_policy.name
     return fs_info
 
 
@@ -939,7 +1291,7 @@ def main():
         dict(gather_subset=dict(default="minimum", type="list", elements="str"))
     )
 
-    module = AnsibleModule(argument_spec, supports_check_mode=False)
+    module = AnsibleModule(argument_spec, supports_check_mode=True)
 
     blade = get_blade(module)
     versions = blade.api_version.list_versions().versions
@@ -950,7 +1302,8 @@ def main():
                 MIN_REQUIRED_API_VERSION
             )
         )
-
+    if not module.params["gather_subset"]:
+        module.params["gather_subset"] = ["minimum"]
     subset = [test.lower() for test in module.params["gather_subset"]]
     valid_subsets = (
         "all",
@@ -967,7 +1320,10 @@ def main():
         "arrays",
         "replication",
         "policies",
+        "accounts",
         "admins",
+        "ad",
+        "kerberos",
     )
     subset_test = (test in valid_subsets for test in subset)
     if not all(subset_test):
@@ -978,8 +1334,9 @@ def main():
 
     info = {}
 
+    api_version = blade.api_version.list_versions().versions
     if "minimum" in subset or "all" in subset:
-        info["default"] = generate_default_dict(blade)
+        info["default"] = generate_default_dict(module, blade)
     if "performance" in subset or "all" in subset:
         info["performance"] = generate_perf_dict(blade)
     if "config" in subset or "all" in subset:
@@ -993,26 +1350,42 @@ def main():
     if "subnets" in subset or "all" in subset:
         info["subnet"] = generate_subnet_dict(blade)
     if "filesystems" in subset or "all" in subset:
-        info["filesystems"] = generate_fs_dict(blade)
+        info["filesystems"] = generate_fs_dict(module, blade)
     if "admins" in subset or "all" in subset:
-        info["admins"] = generate_admin_dict(blade)
+        info["admins"] = generate_admin_dict(module, blade)
     if "snapshots" in subset or "all" in subset:
         info["snapshots"] = generate_snap_dict(blade)
     if "buckets" in subset or "all" in subset:
-        info["buckets"] = generate_bucket_dict(blade)
-    api_version = blade.api_version.list_versions().versions
+        info["buckets"] = generate_bucket_dict(module, blade)
     if POLICIES_API_VERSION in api_version:
         if "policies" in subset or "all" in subset:
             info["policies"] = generate_policies_dict(blade)
+            info["snapshot_policies"] = generate_policies_dict(blade)
     if REPLICATION_API_VERSION in api_version:
         if "arrays" in subset or "all" in subset:
-            info["arrays"] = generate_array_conn_dict(blade)
+            info["arrays"] = generate_array_conn_dict(module, blade)
         if "replication" in subset or "all" in subset:
             info["file_replication"] = generate_file_repl_dict(blade)
-            info["bucket_replication"] = generate_bucket_repl_dict(blade)
+            info["bucket_replication"] = generate_bucket_repl_dict(module, blade)
             info["snap_transfers"] = generate_snap_transfer_dict(blade)
             info["remote_credentials"] = generate_remote_creds_dict(blade)
             info["targets"] = generate_targets_dict(blade)
+    if MIN_32_API in api_version:
+        # Calls for data only available from Purity//FB 3.2 and higher
+        blade = get_system(module)
+        if "accounts" in subset or "all" in subset:
+            info["accounts"] = generate_object_store_accounts_dict(blade)
+        if "ad" in subset or "all" in subset:
+            info["active_directory"] = generate_ad_dict(blade)
+        if "kerberos" in subset or "all" in subset:
+            info["kerberos"] = generate_kerb_dict(blade)
+        if "policies" in subset or "all" in subset:
+            if SMB_MODE_API_VERSION in api_version:
+                info["access_policies"] = generate_object_store_access_policies_dict(
+                    blade
+                )
+            if NFS_POLICY_API_VERSION in api_version:
+                info["export_policies"] = generate_nfs_export_policies_dict(blade)
 
     module.exit_json(changed=False, purefb_info=info)
 
